@@ -9,15 +9,15 @@ Source: https://github.com/RuedigerVoigt/userprovided
 Released under the Apache License 2.0
 """
 
-from userprovided import cloud
+from userprovided import _version
 from userprovided import date
-from userprovided import hash
+from userprovided import hash  # Deprecated / logs warning until v 1.0
+from userprovided import hashing
 from userprovided import mail
 from userprovided import parameters
-from userprovided import port
 from userprovided import url
 
 
-name = "userprovided"
-__version__ = "0.8.0"
+NAME = "userprovided"
+__version__ = _version.__version__
 __author__ = "Rüdiger Voigt"
