@@ -10,8 +10,11 @@ Released under the Apache License 2.0
 """
 
 import logging
+import re
 from typing import Optional, Union
 
+from userprovided import err
+
 
 def convert_to_set(convert_this: Union[list, set, str, tuple]) -> set:
     """ Convert a string, a tuple, or a list into a set
@@ -90,6 +93,40 @@ def validate_dict_keys(dict_to_check: dict,
     return True
 
 
+def keys_neither_none_nor_empty(dict_to_check: dict) -> bool:
+    """Checks if all keys in the provided dictionary are neither None, nor
+       have an empty value (like an empty string (including whitespace only)
+       or a dict/list/set/tuple without elements).
+
+       Does ignore value types other than dict/list/set/str/tuple.
+
+       Raises Value Error if you provide something else than a dictionary,
+       or if its is completely empty."""
+
+    if not isinstance(dict_to_check, dict):
+        raise ValueError('This is not a dictionary')
+    if len(dict_to_check) == 0:
+        raise ValueError('This dictionary is empty')
+
+    def error_found() -> None:
+        logging.error("Dictionary contains key that is either empty or None!")
+
+    for _, value in dict_to_check.items():
+        if value is None:
+            error_found()
+            return False
+        if isinstance(value, str):
+            if len(value.strip()) == 0:
+                error_found()
+                return False
+        if isinstance(value, (dict, list, set, tuple)):
+            if len(value) == 0:
+                error_found()
+                return False
+
+    return True
+
+
 def numeric_in_range(parameter_name: str,
                      given_value: Union[int, float],
                      minimum_value: Union[int, float],
@@ -106,10 +143,12 @@ def numeric_in_range(parameter_name: str,
             raise ValueError('Value must be numeric.')
 
     if minimum_value > maximum_value:
-        raise ValueError("Minimum must not be larger than maximum value.")
+        raise err.ContradictoryParameters(
+            "Minimum must not be larger than maximum value.")
 
     if fallback_value < minimum_value or fallback_value > maximum_value:
-        raise ValueError("Fallback value outside the allowed range.")
+        raise err.ContradictoryParameters(
+            "Fallback value outside the allowed range.")
 
     if given_value < minimum_value:
         msg = (f"Value of {parameter_name} is below the minimum allowed." +
@@ -144,6 +183,20 @@ def int_in_range(parameter_name: str,
                                 fallback_value))
 
 
+def is_port(port_number: int) -> bool:
+    """Check if the number provided is valid as a TCP/UDP port
+       (i.e an integer in the range from 0 to 65535)."""
+
+    if not isinstance(port_number, int):
+        raise ValueError('Port has to be an integer.')
+
+    if 0 < port_number < 65536:
+        logging.debug('Port within range')
+        return True
+    logging.error('Port not within valid range from 0 to 65535')
+    return False
+
+
 def string_in_range(string_to_check: str,
                     minimum_length: int,
                     maximum_lenght: int,
@@ -153,7 +206,7 @@ def string_in_range(string_to_check: str,
        The strip() can be turned off. """
 
     if minimum_length > maximum_lenght:
-        raise ValueError("Minimum must not be larger than maximum value.")
+        raise err.ContradictoryParameters("Minimum must not be larger than maximum value.")
     enforce_boolean(strip_string)
 
     if strip_string:
@@ -167,6 +220,47 @@ def string_in_range(string_to_check: str,
     return True
 
 
+def is_aws_s3_bucket_name(bucket_name: str) -> bool:
+    """Returns True if bucket name is well-formed for AWS S3 buckets
+
+    Applying the rules set here:
+    https://docs.aws.amazon.com/AmazonS3/latest/dev/BucketRestrictions.html
+    """
+
+    # Lengthy code which could be written as a single regular expression.
+    # However written in this way to provide useful error messages.
+    if len(bucket_name) < 3:
+        logging.error(
+            'Any AWS bucket name has to be at least 3 characters long.')
+        return False
+    if len(bucket_name) > 63:
+        logging.error(
+            'The AWS bucket name exceeds the maximum length of 63 characters.')
+        return False
+    if not re.match(r"^[a-z0-9\-\.]*$", bucket_name):
+        logging.error('The AWS bucket name contains invalid characters.')
+        return False
+    if re.match(r"[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}",
+                bucket_name):
+        # Check if the bucket name resembles an IPv4 address.
+        # No need to check IPv6 as the colon is not an allowed character.
+        logging.error('An AWS must not resemble an IP address.')
+        return False
+    if re.match(r"([a-z0-9][a-z0-9\-]*[a-z0-9]\.)*[a-z0-9][a-z0-9\-]*[a-z0-9]",
+                bucket_name):
+        # Must start with a lowercase letter or number
+        # Bucket names must be a series of one or more labels.
+        # Adjacent labels are separated by a single period (.).
+        # Each label must start and end with a lowercase letter or a number.
+        # => Adopted the answer provided by Zak (zero or more labels
+        # followed by a dot) found here:
+        # https://stackoverflow.com/questions/50480924
+        return True
+
+    logging.error('Invalid AWS bucket name.')
+    return False
+
+
 def enforce_boolean(parameter_value: bool,
                     parameter_name: Optional[str] = None) -> None:
     """Raise a ValueError if the parameter is not of type bool."""
